# Classification with R
# --------------------

library(rpart)
library(rattle)

# load the dataset
iris = read.csv('iris.csv', header = TRUE, sep=',')

iris.train <- c(sample(1:150,75))
iris.test <- iris[-iris.train,]

# Decision tree
# --------------------
iris.dtree <- rpart(class ~ ., data=iris, subset=iris.train)
iris.dtree <- rpart(class ~ sepal.length + sepal.width + petal.length + petal.width, data=iris)

print(iris.dtree)
printcp(iris.dtree) # display the results 
plotcp(iris.dtree) # visualize cross-validation results 
summary(iris.dtree) # detailed summary of splits
table(predict(iris.dtree,iris[-iris.train,], type='class'), iris[-iris.train,'class'])
table(predict(iris.dtree, iris.test[1:4], type='class'), iris.test[,5])

# plot tree 
plot(iris.dtree)
plot(iris.dtree, uniform=TRUE, main="Classification Tree for Iris")
text(iris.dtree, use.n=TRUE, all=TRUE, cex=.8)
post(iris.dtree, file = "tree.ps",  title="Classification Tree for Iris")

# prune the tree
iris.ptree<- prune(iris.dtree, cp=iris.dtree$cptable[which.min(iris.dtree$cptable[,"xerror"]),"CP"])
plot(iris.ptree, uniform=TRUE, main="Pruned classification Tree for Iris")

# drawTreeNodes(iris.dtree)
# table(predict(iris.dtree,iris[-iris.train,], type='class'), iris[-iris.train,'class'])


# Naive bayes classifier
# --------------------
indexes_train <- c(sample(1:150,75))
iris.train <- iris[indexes_train,]
iris.test <- iris[-indexes_train,]

library(e1071)
iris.nbayes <- naiveBayes(class~.,data=iris.train)
table(predict(iris.nbayes, iris.test[1:4]), iris.test[,5])
print(iris.nbayes)


# Support Vector Machines
# --------------------
iris.svm <- svm(class ~ ., data=iris.train)
table(predict(iris.svm, iris.test[1:4]), iris.test[,5])
print(iris.svm)


# Regression
# --------------------
iris.lm <- lm(iris$petal.width ~ iris$sepal.length + iris$sepal.width + iris$petal.length)
summary(iris.lm)

iris.lm <- lm(iris$petal.width ~ iris$petal.length + I(iris$petal.length^2))
summary(iris.lm$w)


# Random Forest prediction of Kyphosis data
# --------------------
library(randomForest)
iris.rf <- randomForest(class~., data=iris.train)
print(iris.rf) # view results 
importance(iris.rf) # importance of each predictor
table(predict(iris.rf, iris.test[1:4]), iris.train[,5])



# Nearest Neighbor Classifiers
# --------------------
iris.knn <- knn(iris.train[,1:4],iris.test[,1:4], iris.train[,5],k=3)
table(iris.knn,iris.test[,5])


# Neural Networks
# --------------------
library(neuralnet)
x <- iris.train
x$class <- as.numeric(x$class)
iris.nn <- neuralnet(x$class + x$sepal.length ~ x$sepal.width + x$petal.length + x$petal.width, x, hidden=c(3))
plot(iris.nn)
y <- iris.test
y <- y[-5]
y <- y[-1]
y.out <- compute(iris.nn,y)
y.sqerr <- (y[1] - y.out$net.result[,2])^2

