# Frequent pattern with R
# --------------------

library(arules)

# dumy example
baskets <- list(c("a","b","c"), c("a","d","e"), c("b","c","d"), c("a","b","c","d"), c("b","c"), c("a","b","d"), c("d","e"), c("a","b","c","d"), c("c","d","e"), c("a","b","c"))
rules <- apriori(baskets, parameter = list(supp=0.1, conf=0.8, target="rules"))
inspect(rules)
summary(rules)

# read transactional data
baskets <- read.transactions("baskets.csv", format="basket", sep=",")
inspect(baskets)
summary(baskets)

# itemsets
# options "frequent itemsets", "maximally frequent itemsets", "closed frequent itemsets"
itemsets <- apriori(baskets, parameter = list(minlen=2, supp=0.05, confidence=0.8, target="frequent itemsets"))
inspect(rules)
summary(rules)

# rules
rules <- apriori(recipies, parameter = list(minlen=2, supp=0.05, confidence=0.8, target="rules"))
inspect(rules)
summary(rules)


# from classic dataset to transactional
adult = read.csv('adult.csv', header = TRUE, sep=',')
dim(adult)
adult[1:2,]

## remove attributes
adult[["fnlwgt"]] <- NULL
adult[["education.num"]] <- NULL

## map metric attributes
adult[[ "age"]] <- ordered(cut(adult[[ "age"]], c(15,25,45,65,100)), labels = c("Young", "Middle-aged", "Senior", "Old"))
adult[[ "hours.per.week"]] <- ordered(cut(adult[[ "hours.per.week"]], c(0,25,40,60,168)), labels = c("Part-time", "Full-time", "Over-time", "Workaholic"))
adult[[ "capital.gain"]] <- ordered(cut(adult[[ "capital.gain"]], c(-Inf,0,median(adult[[ "capital.gain"]][adult[[ "capital.gain"]]>0]), Inf)), labels = c("None", "Low", "High"))
adult[[ "capital.loss"]] <- ordered(cut(adult[[ "capital.loss"]], c(-Inf,0, median(adult[[ "capital.loss"]][adult[[ "capital.loss"]]>0]), Inf)), labels = c("None", "Low", "High"))


# transformation
adult_transaction <- as(adult, "transactions")
inspect(adult_transaction)
summary(adult_transaction)

# itemsets
# options "frequent itemsets", "maximally frequent itemsets", "closed frequent itemsets"
itemsets <- apriori(adult_transaction, parameter = list(minlen=2, supp=0.1, confidence=0.8, target="frequent"))
inspect(itemsets)
summary(itemsets)
itemFrequency(items(itemsets))['age=Young']
itemFrequency(items(itemsets))['workclass= Private']

# rules
rules <- apriori(adult_transaction, parameter = list(minlen=2, supp=0.1, confidence=0.8, target="rules"))
inspect(rules)
summary(rules)

# support
sp <- crossTable(adult_transaction, measure="support", sort=TRUE)
sp[1:5,1:5]

# lift
lift <- crossTable(adult_transaction, measure="lift", sort=TRUE)
lift[1:5,1:5]
